//
//  ISViewController.m
//  ExpandableTextCell
//
//  Created by Simon Liu on 17/6/8.
//  Copyright © 2017年 xzliu. All rights reserved.
//

#import "ISViewController.h"
#import "ISExpandableTextCell.h"
#import <ISIDReaderPreviewSDK/ISIDReaderPreviewSDK.h>
#import <ISOpenSDKFoundation/ISOpenSDKFoundation.h>
#import "ISZoomScrollView.h"

#define IS_DEVICE_HEIGHT    [[UIScreen mainScreen] bounds].size.height
#define IS_DEVICE_WIDTH     [[UIScreen mainScreen] bounds].size.width

#define COLOR_WITH_HEX(hexValue) [UIColor colorWithRed:((float)((hexValue & 0xFF0000) >> 16)) / 255.0 green:((float)((hexValue & 0xFF00) >> 8)) / 255.0 blue:((float)(hexValue & 0xFF)) / 255.0 alpha:1.0f]


@interface ISViewController ()<ISExpandableTableViewDelegate>
{
    CGFloat _cellHeight[1000];
    BOOL _loadImageBOOL;
    NSArray *_infoImageArray;
}

@property (nonatomic, strong) NSMutableArray *cellData;
@property (nonatomic, strong) NSMutableArray *cellDataKey;
@property (strong, nonatomic) UIImage *cardResultTypeImage;
@property (strong, nonatomic) UIImage *headImage;
@property (strong, nonatomic) UIImage *typeOriginImage;
@property (strong, nonatomic) ISZoomScrollView *zoomScrollView ;
@property (strong, nonatomic) NSString *ocrName;
@property (strong, nonatomic) UIImage *cardNumImage;
@property (strong, nonatomic) UIImageView *headerImageView;
@property (strong, nonatomic) UIButton *cardNumBtn;
@property (strong, nonatomic) UIView *cardNumMaskView;
@end

@implementation ISViewController

- (instancetype)initWithDictionary:(NSDictionary *)info loadImageBool:(BOOL)loadImageBOOL
{
    self = [super init];
    if (self != nil) {

        _loadImageBOOL=loadImageBOOL;
        [self showISOCRResult:info];
    }
    return self;
}

#warning 获取OCR输出结果，请查看返回的对应数据结构体；
- (void)showISOCRResult:(NSDictionary *)resultInfo
{
    NSMutableString *contentString = [NSMutableString stringWithCapacity:0];
    self.cellData =[[NSMutableArray alloc] init];
    self.cellDataKey =[[NSMutableArray alloc] init];
    
    NSDictionary *infoDict = resultInfo[kOpenSDKCardResultTypeCardItemInfo];

    ISOpenSDKIDCardType type = [resultInfo[kOpenSDKCardResultTypeCardType] integerValue];
    switch (type) {
        case ISOpenSDKIDCardTypeSecondMain:
        {
            NSLog(@"正面");
        }
            break;
        case ISOpenSDKIDCardTypeFirst:
        {
            NSLog(@"第一代身份证");
        }
            break;
        case ISOpenSDKIDCardTypeSecondCover:
        {
            NSLog(@"背面");
        }
            break;
            
        default:
            break;
    }
    
    
    NSString *cardName = resultInfo[kOpenSDKCardResultTypeCardName];
    UIImage *cardIDNumberImage = resultInfo[kIDCardNumberImage];
    UIImage *headImage =resultInfo[kOpenSDKCardResultTypeHeadImage];
    UIImage *cardCropImage =resultInfo[kOpenSDKCardResultTypeImage];
    UIImage *cardTypeOriginImage = resultInfo[kOpenSDKCardResultTypeOriginImage];
    
    self.cardResultTypeImage = cardCropImage;
    self.typeOriginImage = cardTypeOriginImage;
    self.headImage = headImage;
    self.cardNumImage = cardIDNumberImage;
    self.ocrName = cardName;
    
    [contentString appendFormat:@"%@%@\n", @"身份号截图", cardIDNumberImage];
    [contentString appendFormat:@"%@%@\n", @"人物头像", headImage];
    [contentString appendFormat:@"%@%@\n", @"身份证完整切边图片", cardCropImage];
    [contentString appendFormat:@"%@%@\n", @"身份证原图", cardTypeOriginImage];
    
    if (cardName != nil)
    {
        [contentString appendFormat:@"%@\n", cardName];
    }
    
    
    
    if ([infoDict isKindOfClass:[NSDictionary class]])
    {
        NSString *cardName = [infoDict valueForKey:kCardItemName];
        if (cardName != nil)
        {
            [self.cellDataKey addObject:@"姓名"];
            [self.cellData addObject:cardName];
            [contentString appendFormat:@"%@%@\n", NSLocalizedString(@"  Name:", @"姓名："), cardName];
        }
        NSString *sex = [infoDict valueForKey:kCardItemGender];
        if (sex != nil)
        {
            [self.cellDataKey addObject:@"性别"];
            [self.cellData addObject:sex];
            [contentString appendFormat:@"%@%@\n", NSLocalizedString(@"  Gender:", @"性别："), sex];
        }
        NSString *nation = [infoDict valueForKey:kCardItemNation];
        if (nation != nil)
        {
            [self.cellDataKey addObject:@"民族"];
            [self.cellData addObject:nation];
            [contentString appendFormat:@"%@%@\n", NSLocalizedString(@"  Nation:", @"民族："), nation];
        }
        NSString *birthday = [infoDict valueForKey:kCardItemBirthday];
        if (birthday != nil)
        {
            [self.cellDataKey addObject:@"出生日期"];
            [self.cellData addObject:birthday];
            [contentString appendFormat:@"%@%@\n", NSLocalizedString(@"  Birthday:", @"出生日期："), birthday];
        }
        NSString *address = [infoDict valueForKey:kCardItemAddress];
        if (address != nil)
        {
            [self.cellDataKey addObject:@"地址"];
            [self.cellData addObject:address];
            [contentString appendFormat:@"%@%@\n", NSLocalizedString(@"  Address:", @"地址："), address];
        }
        NSString *idNumber = [infoDict valueForKey:kCardItemIDNumber];
        if (idNumber != nil)
        {
            [self.cellDataKey addObject:@"身份证号"];
            [self.cellData addObject:idNumber];
            [contentString appendFormat:@"%@%@\n", NSLocalizedString(@"  ID Card No:", @"身份证号码："), idNumber];
        }
        NSString *issueAuthority = [infoDict valueForKey:kCardItemIssueAuthority];
        if (issueAuthority != nil)
        {
            [self.cellDataKey addObject:@"签发机关"];
            [self.cellData addObject:issueAuthority];
            [contentString appendFormat:@"%@%@\n", NSLocalizedString(@"  Issue Authority:", @"签发机关："), issueAuthority];
        }
        NSString *validity = [infoDict valueForKey:kCardItemValidity];
        if (validity != nil)
        {
            [self.cellDataKey addObject:@"有效期限"];
            [self.cellData addObject:validity];
            [contentString appendFormat:@"%@%@\n", NSLocalizedString(@"  Issue Authority:", @"有效期限："), validity];
        }
        NSString *completeness = @"";
        ISIDCardImageCompletenessType type = [infoDict[kIDCardImageCompletenessType] integerValue];
        switch (type)
        {
            case ISIDCardImageCompletenessTypeComplete:
                completeness = @"完整";
                break;
            case ISIDCardImageCompletenessTypeImageCover:
                completeness = @"图像有遮挡";
                break;
            case ISIDCardImageCompletenessTypePhotoCover:
                completeness = @"正面人物头像质量不好";
                break;
            case ISIDCardImageCompletenessTypeNationalEmblemCover:
                completeness = @"国徽有遮挡";
                break;
            default:
                break;
        }
        if (completeness.length > 0)
        {
            [self.cellDataKey addObject:@"是否完整"];
            [self.cellData addObject:completeness];
            [contentString appendFormat:@"%@%@\n", NSLocalizedString(@"  Completeness:", @"身份证是否完整："), completeness];
        }
        NSInteger isColorImageValue = [infoDict[kIDCardColorImage] integerValue];
        NSString *isColorImage = isColorImageValue > 0 ? @"是" : @"否";
        [self.cellDataKey addObject:@"是否彩色"];
        [self.cellData addObject:isColorImage];
        [contentString appendFormat:@"%@%@",NSLocalizedString(@"  Color:", @"身份证是否为彩色："), isColorImage];
    }
    NSLog(@"ISOCR Result :\n%@",contentString);
    
}


- (void)viewDidLoad
{
    [super viewDidLoad];
    
    //字体颜色
    self.navigationController.navigationBar.tintColor = [UIColor grayColor];
    self.navigationController.navigationBar.backgroundColor = [UIColor whiteColor];
    self.tableView.backgroundColor = [UIColor colorWithRed:241.0/255.0 green:241.0/255.0 blue:241.0/255.0 alpha:1];
    self.tableView.separatorColor =[UIColor colorWithRed:241.0/255.0 green:241.0/255.0 blue:241.0/255.0 alpha:1];
    self.tableView.separatorInset = UIEdgeInsetsMake(0, 15, 0, 15);//分割线缩进 两端距离
    
    if (_loadImageBOOL==NO) {
        _infoImageArray = [[NSArray alloc] init];
    }else{
        _infoImageArray = @[self.cardResultTypeImage,self.typeOriginImage];
        
    }
    
    self.navigationItem.title = [NSString stringWithFormat:@"%@ 识别结果",[self.ocrName isEqualToString:@"第二代身份证"]?@"第二代身份证正面":self.ocrName];
    UITapGestureRecognizer * tapGesture = [[UITapGestureRecognizer alloc]initWithTarget:self action:@selector(dismissKeyBoard)];
    tapGesture.cancelsTouchesInView = NO;
    [self.tableView addGestureRecognizer:tapGesture];
    UIBarButtonItem *leftBarItem =  [[UIBarButtonItem alloc] initWithImage:[UIImage imageNamed:@"IS_Back"] style:UIBarButtonItemStyleDone target:self action:@selector(clickLeftBarButtonItem)];
    self.navigationItem.leftBarButtonItem = leftBarItem;
    
    UIView *header = [[UIView alloc] initWithFrame:CGRectMake(0, 0, IS_DEVICE_WIDTH, IS_DEVICE_HEIGHT*2/7)];
    [self setUpZoomScrollView:header];
    // 设置header
    self.tableView.tableHeaderView = header;
    
}

- (void)setUpZoomScrollView:(UIView *)addView
{
    self.zoomScrollView = [ISZoomScrollView roundScrollViewWithFrame:addView.frame imageArr:_infoImageArray timerWithTimeInterval:5 imageClick:^(NSInteger imageIndex) {
    }];
    [addView addSubview:self.zoomScrollView];
}


- (void)clickLeftBarButtonItem
{
    [self dismissViewControllerAnimated:YES completion:nil];
    
    [self.zoomScrollView removeFromSuperview];
    
    
}

-(void)dismissKeyBoard{
    [self.view endEditing:YES];
}

#pragma mark - Table View Data Source

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    
    return self.cellData.count;
    
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    //    NSString *CellIdentifier = [NSString stringWithFormat:@"Cell%ld%ld", (long)[indexPath section], (long)[indexPath row]];//以indexPath来唯一确定cell
    static NSString *CellIdentifier = @"StCellIdentifier";
    ISExpandableTextCell *cell = [tableView expandableTextCellWithId:CellIdentifier];
    
    if (indexPath.row == 0 || indexPath.row == 1 || indexPath.row == 2 )
    {
        
        if (indexPath.row == 0)
        {
            self.headerImageView = [[UIImageView alloc] init];
            self.headerImageView.frame = CGRectMake(self.view.frame.size.width - 97 - 15, self.tableView.tableHeaderView.frame.size.height + 9, 97, 112);
            self.headerImageView.image = self.headImage;
            [tableView addSubview:self.headerImageView];
            
            if (![self.ocrName isEqualToString:@"第二代身份证背面"])
            {
                cell.textView.frame = CGRectMake(80, 5, self.view.frame.size.width - 85 - 115, cell.frame.size.height-5);
                
            }
            
        }else if (indexPath.row ==1)
        {
            if (![self.ocrName isEqualToString:@"第二代身份证背面"])
            {
                cell.textView.frame = CGRectMake(80, 5, self.view.frame.size.width - 85 - 115, cell.frame.size.height-5);
                
            }
            
        }else
        {
            if (![self.ocrName isEqualToString:@"第二代身份证背面"])
            {
                cell.textView.frame = CGRectMake(80, 5, self.view.frame.size.width - 85 - 115, cell.frame.size.height-5);
                
            }
        }
        if (![self.ocrName isEqualToString:@"第二代身份证背面"])
        {
            if (indexPath.row != 2)
            {
                cell.separatorInset = UIEdgeInsetsMake(0, 15, 0, 125);
            }
        }
        
        
        
    }else if (indexPath.row == 5)
    {
        if (self.cardNumBtn == nil)
        {
            self.cardNumBtn = [[UIButton alloc] init];
            self.cardNumBtn.frame = CGRectMake(self.view.frame.size.width - 44 - 15, 5, 44, 34);
            [self.cardNumBtn setImage:[UIImage imageNamed:@"IS_OCR_Search.png"] forState:UIControlStateNormal];
            [self.cardNumBtn addTarget:self action:@selector(clickCardNumberButtonMethod) forControlEvents:UIControlEventTouchUpInside];
            [cell.contentView addSubview:self.cardNumBtn];
            cell.textView.frame = CGRectMake(80, 5, self.view.frame.size.width - 85 - 52, cell.frame.size.height-5);
            
        }
        
    }
    
    cell.leftTextView.text = self.cellDataKey[indexPath.row];
    cell.textView.text =[self.cellData objectAtIndex:indexPath.row];
    
    return cell;
}

- (void)clickCardNumberButtonMethod
{
    NSLog(@"click IDNumber Image");
    
    [self.view endEditing:YES];
    //    _scrollViewResult.contentOffset = CGPointMake(0, -64);
    self.cardNumMaskView = [[UIView alloc] init];
    self.cardNumMaskView.frame = self.view.window.frame;
    self.cardNumMaskView.backgroundColor = [UIColor colorWithRed:0.0f/255.0f green:0.0f/255.0f blue:0.0f/255.0f alpha:0.5];
    [[UIApplication sharedApplication].keyWindow addSubview:self.cardNumMaskView];
    
    UITapGestureRecognizer *tapGesture = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(clickWindowsBackGroudViewTapGesture)];
    [self.cardNumMaskView addGestureRecognizer:tapGesture];
    
    UIImageView *cardNumImageView = [[UIImageView alloc] init];
    
    CGFloat x_float = self.view.frame.size.width/8;
    cardNumImageView.frame = CGRectMake(x_float, 350, self.view.frame.size.width - x_float * 2, 36);
    cardNumImageView.image = self.cardNumImage;
    [self.cardNumMaskView addSubview:cardNumImageView];
    
}
- (void)clickWindowsBackGroudViewTapGesture
{
    [self.cardNumMaskView removeFromSuperview];
}

#pragma mark - Table View Delegate
- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath
{
    
    NSString* text = self.cellData[indexPath.row];
    CGSize constraint = CGSizeMake(self.view.frame.size.width-135, MAXFLOAT);
    
    NSAttributedString* attributedText = [[NSAttributedString alloc]initWithString:text attributes:@{NSFontAttributeName:[UIFont systemFontOfSize:14]}];
    
    CGRect rect = [attributedText boundingRectWithSize:constraint options:NSStringDrawingUsesLineFragmentOrigin context:nil];
    
    CGSize size = rect.size;
    
    CGFloat height = MAX(size.height+26.5, _cellHeight[indexPath.row]);
    
    return height;
}

- (void)tableView:(UITableView *)tableView updatedHeight:(CGFloat)height atIndexPath:(NSIndexPath *)indexPath
{
    
    _cellHeight[indexPath.row] = height;
    
}

- (void)tableView:(UITableView *)tableView updatedText:(NSString *)text atIndexPath:(NSIndexPath *)indexPath
{
    
    [_cellData replaceObjectAtIndex:indexPath.row withObject:text];
    
}

@end
