//
//  ISIDReaderResultViewController.m
//  ISIDReaderPreview
//
//  Created by Simon liu on 17/2/7.
//  Copyright (c) 2017年 xzliu. All rights reserved.
//

#import "ISIDReaderResultViewController.h"
#import <ISIDReaderPreviewSDK/ISIDReaderPreviewSDK.h>
//#import "ISIDCardReaderController.h"


@interface ISIDReaderResultViewController ()

@property (weak, nonatomic) IBOutlet UILabel *titleLabel;
@property (weak, nonatomic) IBOutlet UIImageView *originalImageView;
@property (weak, nonatomic) IBOutlet UIImageView *cardImageView;
@property (weak, nonatomic) IBOutlet UIImageView *IDNumberImageView;
@property (weak, nonatomic) IBOutlet UIImageView *headImageView;
@property (weak, nonatomic) IBOutlet UITextView *itemInfoTextView;

@property (copy, nonatomic) NSString *cardName;
@property (strong, nonatomic) UIImage *originalImage;
@property (strong, nonatomic) UIImage *cardImage;
@property (strong, nonatomic) UIImage *numberImage;
@property (strong, nonatomic) UIImage *headImage;
@property (strong, nonatomic) NSDictionary *infoDic;
@property (nonatomic, assign) NSInteger rotateAngle;

@end

@implementation ISIDReaderResultViewController

- (instancetype)initWithDictionary:(NSDictionary *)info
{
    self = [super init];
    if (self != nil)
    {
        self.cardName = info[kOpenSDKCardResultTypeCardName];
        self.originalImage = info[kOpenSDKCardResultTypeOriginImage];
        self.cardImage = info[kOpenSDKCardResultTypeImage];
        self.numberImage = info[kIDCardNumberImage];
        self.headImage = info[kOpenSDKCardResultTypeHeadImage];
        self.rotateAngle = [info[kOpenSDKCardResultTypeCardRotate] integerValue];
        self.infoDic = info[kOpenSDKCardResultTypeCardItemInfo];
        NSArray *cardNumberPointsInOriginImage = [info objectForKey:kIDCardNumberPointsInOriginImage];
        NSValue *cardNumberRectInCropedImage = [info objectForKey:kIDCardNumberRectInCroppedImage];//CGRect value
        NSLog(@"cardNumberPointsInOriginImage: %@\ncardNumberRectInCropedImage: %@", cardNumberPointsInOriginImage, cardNumberRectInCropedImage);
    }
    return self;
}

- (IBAction)okbuttonClick:(id)sender
{
    [self dismissViewControllerAnimated:YES completion:^{
        
    }];
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    self.originalImageView.image = self.originalImage;
    self.originalImageView.transform = CGAffineTransformMakeRotation(M_PI*self.rotateAngle/180);
    self.cardImageView.image = self.cardImage;
    self.IDNumberImageView.image = self.numberImage;
    self.headImageView.image = self.headImage;
    
    NSMutableString *contentString = [NSMutableString stringWithCapacity:0];
    if (self.cardName != nil)
    {
        [contentString appendFormat:@"%@\n", self.cardName];
    }
    if ([self.infoDic isKindOfClass:[NSDictionary class]])
    {
        NSString *cardName = [self.infoDic valueForKey:kCardItemName];
        if (cardName != nil)
        {
            [contentString appendFormat:@"%@%@\n", NSLocalizedString(@"ID Card Name:", @"姓名："), cardName];
        }
        NSString *sex = [self.infoDic valueForKey:kCardItemGender];
        if (sex != nil)
        {
            [contentString appendFormat:@"%@%@\n", NSLocalizedString(@"Gender:", @"性别："), sex];
        }
        NSString *nation = [self.infoDic valueForKey:kCardItemNation];
        if (nation != nil)
        {
            [contentString appendFormat:@"%@%@\n", NSLocalizedString(@"Nation:", @"民族："), nation];
        }
        NSString *birthday = [self.infoDic valueForKey:kCardItemBirthday];
        if (birthday != nil)
        {
            [contentString appendFormat:@"%@%@\n", NSLocalizedString(@"Birthday:", @"出生日期："), birthday];
        }
        NSString *address = [self.infoDic valueForKey:kCardItemAddress];
        if (address != nil)
        {
            [contentString appendFormat:@"%@%@\n", NSLocalizedString(@"ID Card Address:", @"地址："), address];
        }
        NSString *idNumber = [self.infoDic valueForKey:kCardItemIDNumber];
        if (idNumber != nil)
        {
            [contentString appendFormat:@"%@%@\n", NSLocalizedString(@"ID Card Number:", @"身份证号码："), idNumber];
        }
        NSString *issueAuthority = [self.infoDic valueForKey:kCardItemIssueAuthority];
        if (issueAuthority != nil)
        {
            [contentString appendFormat:@"%@%@\n", NSLocalizedString(@"ID Card Issue Authority:", @"签发机关："), issueAuthority];
        }
        NSString *validity = [self.infoDic valueForKey:kCardItemValidity];
        if (validity != nil)
        {
            [contentString appendFormat:@"%@%@\n", NSLocalizedString(@"ID Card Validity:", @"有效期限："), validity];
        }
        NSString *completeness = @"";
        ISIDCardImageCompletenessType type = [[self.infoDic objectForKey:kIDCardImageCompletenessType] integerValue];
        switch (type)
        {
            case ISIDCardImageCompletenessTypeComplete:
                completeness = @"完整";
                break;
            case ISIDCardImageCompletenessTypeImageCover:
                completeness = @"图像有遮挡";
                break;
            case ISIDCardImageCompletenessTypePhotoCover:
                completeness = @"正面人物头像质量不好";
                break;
            case ISIDCardImageCompletenessTypeNationalEmblemCover:
                completeness = @"国徽有遮挡";
                break;
            default:
                break;
        }
        if (completeness.length > 0)
        {
            [contentString appendFormat:@"%@%@\n", NSLocalizedString(@"ID Card Completeness:", @"身份证是否完整："), completeness];
        }
        
        NSInteger isColorImageValue = [self.infoDic[kIDCardColorImage] integerValue];
        NSString *isColorImage = isColorImageValue > 0 ? @"是" : @"否";
        [contentString appendFormat:@"%@%@",NSLocalizedString(@"ID Card Color:", @"身份证是否为彩色："), isColorImage];
    }
    
    self.itemInfoTextView.text = contentString;
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (UIImage *) croppedImage:(UIImage*)image withRect: (CGRect) bounds
{
    CGImageRef imageRef = CGImageCreateWithImageInRect([image CGImage], bounds);
    UIImage *croppedImage = [UIImage imageWithCGImage:imageRef];
    CGImageRelease(imageRef);
    return croppedImage;
}

@end
