//
//  ViewController.m
//  ISIDReaderPreview
//
//  Created by Simon liu on 17/2/7.
//  Copyright (c) 2017年 xzliu. All rights reserved.
//

#import "ViewController.h"
#import <AVFoundation/AVFoundation.h>
#import <ISIDReaderPreviewSDK/ISIDReaderPreviewSDK.h>
#import <ISOpenSDKFoundation/ISOpenSDKFoundation.h>
#import <CoreGraphics/CoreGraphics.h>
#import "MBProgressHUD.h"
#import "ISIDReaderResultViewController.h"

//黄金比例
#define GoldenSectionRation 0.63

typedef NS_ENUM(NSInteger, ISPreviewExpandOrientation)
{
    ISPreviewExpandOrientationPortait,
    ISPreviewExpandOrientationLandscape
};

static const NSString *AVCaptureStillImageIsCapturingStillImageContext = @"AVCaptureStillImageIsCapturingStillImageContext";

@interface ViewController ()<AVCaptureVideoDataOutputSampleBufferDelegate, UIAlertViewDelegate>
@property (nonatomic, strong) AVCaptureSession *captureSession;
@property (nonatomic, strong) AVCaptureVideoPreviewLayer *previewLayer;
@property (nonatomic, strong) AVCaptureVideoDataOutput *videoDataOutput;
@property (nonatomic, strong) dispatch_queue_t videoDataOutputQueue;
@property (nonatomic, strong) AVCaptureStillImageOutput *stillImageOutput;
@property (nonatomic, strong) UIView *borderView;
@property (nonatomic, assign) int index;
@property (nonatomic, strong) CALayer *cardNumberLayer;
@property (nonatomic, strong) CALayer *cardBorderLayer;

@property (nonatomic, assign) CGFloat imageFrameWidth;
@property (nonatomic, assign) CGFloat imageFrameHeight;
@property (nonatomic, assign) ISPreviewExpandOrientation expandOrientation;
@property (nonatomic, assign) BOOL isFirstLaunch;
@property (nonatomic, assign) NSNumber *isLandScaped;
//预览框内尺寸
@property (nonatomic, assign) NSInteger borderWidth;
@property (nonatomic, assign) NSInteger borderHeight;
@property (nonatomic, strong) CALayer *backGroundLayer;
//获取拍摄取景框的尺寸大小，该尺寸的大小是根据像素和频幕尺寸比例进行转换
@property (nonatomic, assign) CGRect borderRectInImage;
//设置默认提示图片
@property (nonatomic, strong) UIImageView *imageView;

@end

@implementation ViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view, typically from a nib.
    self.isFirstLaunch = YES;
    //对屏幕旋转进行监听
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(orientationChanged:) name:UIDeviceOrientationDidChangeNotification object:nil];
}
- (void)orientationChanged:(NSNotification *)noti
{
    [self setupBorderView];
}

- (void)viewDidAppear:(BOOL)animated
{
    [super viewDidAppear:animated];
    
    dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_HIGH, 0), ^
                   {
                       //#error Please fill in your appKey
                       NSString *appKey = @"fda3912a00111c9319232c1dd1-vagfvt";
                       NSString *subAppkey = nil;//reserved for future use
                       [[ISIDCardReaderController sharedISOpenSDKController] constructResourcesWithAppKey:appKey subAppkey:subAppkey finishHandler:^(ISOpenSDKStatus status) {
                           if(status == ISOpenSDKStatusSuccess)
                           {
                               dispatch_async(dispatch_get_main_queue(), ^
                                              {
                                                  [self setupAVCapture];
                                                  [self setupBorderView];
                                                  [self setupOtherThings];
                                                  [self focus];
                                                  if (![self.captureSession isRunning])
                                                  {
                                                      [self.captureSession startRunning];
                                                  }
                                              });
                           }
                           else
                           {
                               NSLog(@"Authorize error");
                           }
                       }];
                       [[ISIDCardReaderController sharedISOpenSDKController] setNeedCompleteness:YES];
                   });
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (void) focus
{
    AVCaptureDevice *device = [AVCaptureDevice defaultDeviceWithMediaType:AVMediaTypeVideo];
    if(device.isFocusPointOfInterestSupported && [device isFocusModeSupported:AVCaptureFocusModeContinuousAutoFocus])
    {
        NSError *error =  nil;
        if([device lockForConfiguration:&error]){
            [device setFocusPointOfInterest:CGPointMake(.5f, .5f)];
            [device setFocusMode:AVCaptureFocusModeContinuousAutoFocus];
            [device unlockForConfiguration];
        }
    }
}
//设置视频流捕获
- (void)setupAVCapture
{
    NSError *error = nil;
    
    AVCaptureSession *session = [AVCaptureSession new];
    //1920*1080 is the suggested size if it is supported by device.
    if ([session canSetSessionPreset:AVCaptureSessionPreset1920x1080])
    {
        NSLog(@"AVCaptureSessionPreset1920x1080");
        [session setSessionPreset:AVCaptureSessionPreset1920x1080];
        self.imageFrameWidth = 1920;
        self.imageFrameHeight = 1080;
    }
    else if ([session canSetSessionPreset:AVCaptureSessionPreset640x480])
    {
        NSLog(@"AVCaptureSessionPreset640x480");
        [session setSessionPreset:AVCaptureSessionPreset640x480];
        self.imageFrameWidth = 640;
        self.imageFrameHeight = 480;
    }
    else if ([session canSetSessionPreset:AVCaptureSessionPreset352x288])
    {
        NSLog(@"AVCaptureSessionPreset352x288");
        [session setSessionPreset:AVCaptureSessionPreset352x288];
        self.imageFrameWidth = 352;
        self.imageFrameHeight = 288;
    }
    else
    {
        NSLog(@"failed  setSessionPreset");
        return;
    }
    self.captureSession = session;
    // Select a video device, make an input
    AVCaptureDevice *device = [AVCaptureDevice defaultDeviceWithMediaType:AVMediaTypeVideo];
    AVCaptureDeviceInput *deviceInput = [AVCaptureDeviceInput deviceInputWithDevice:device error:&error];
    
    if ( [session canAddInput:deviceInput] )
        [session addInput:deviceInput];
    
    // Make a video data output
    self.videoDataOutput = [AVCaptureVideoDataOutput new];
    
    // we want YUV, both CoreGraphics and OpenGL work well with 'YUV'
    NSDictionary *rgbOutputSettings = [NSDictionary dictionaryWithObject:
                                       [NSNumber numberWithInt:kCVPixelFormatType_420YpCbCr8BiPlanarFullRange] forKey:(id)kCVPixelBufferPixelFormatTypeKey];
    [self.videoDataOutput setVideoSettings:rgbOutputSettings];
    [self.videoDataOutput setAlwaysDiscardsLateVideoFrames:YES];
    
    self.videoDataOutputQueue = dispatch_queue_create("VideoDataOutputQueue", DISPATCH_QUEUE_SERIAL);
    [self.videoDataOutput setSampleBufferDelegate:self queue:self.videoDataOutputQueue];
    
    if ([session canAddOutput:self.videoDataOutput])
    {
        [session addOutput:self.videoDataOutput];
    }
    [[self.videoDataOutput connectionWithMediaType:AVMediaTypeVideo] setEnabled:YES];
    
    //屏幕成像预览区域
    self.previewLayer = [[AVCaptureVideoPreviewLayer alloc] initWithSession:session];
    [self.previewLayer setBackgroundColor:[[UIColor blackColor] CGColor]];
    //设置填充模式
    [self.previewLayer setVideoGravity:AVLayerVideoGravityResizeAspect];
    
    CALayer *rootLayer = [self.view layer];
    [rootLayer setMasksToBounds:YES];
    
    //获取当前屏幕尺寸大小
    CGFloat screenWidth = [[UIScreen mainScreen] bounds].size.width;
    CGFloat scrrenHeight = [[UIScreen mainScreen] bounds].size.height;
    
    
    //建立视频预览区域大小
    if (screenWidth / self.imageFrameHeight > scrrenHeight / self.imageFrameWidth)
    {
        CGFloat height = self.imageFrameWidth * screenWidth / self.imageFrameHeight;
        CGRect frameRect = CGRectMake(0, (scrrenHeight - height) / 2, screenWidth, height);
        [self.previewLayer setFrame:frameRect];
        self.expandOrientation = ISPreviewExpandOrientationPortait;
    }
    else
    {
        CGFloat width = self.imageFrameHeight * scrrenHeight / self.imageFrameWidth;
        CGRect frameRect = CGRectMake((screenWidth - width) / 2, 0, width, scrrenHeight);
        [self.previewLayer setFrame:frameRect];
        self.expandOrientation = ISPreviewExpandOrientationLandscape;
    }
    [rootLayer addSublayer:self.previewLayer];
    [session startRunning];
    
}

//设置取景框
- (void)setupBorderView
{
    if (self.previewLayer == nil)
    {
        return;
    }
    CGSize screenSize = [[UIScreen mainScreen] bounds].size;
    CGFloat screenWidth = screenSize.width;
    CGFloat screenHeight = screenSize.height;
    //监听Home键方位
    UIDeviceOrientation orientation = [UIDevice currentDevice].orientation;
    BOOL isLandScaped = YES;
    //设置旋转角度
    CGFloat angle = 0;
    if (orientation == UIDeviceOrientationPortrait || orientation == UIDeviceOrientationPortraitUpsideDown)
    {
        isLandScaped = NO;
    }
    else if (orientation == UIDeviceOrientationFaceUp && self.isFirstLaunch)
    {
        isLandScaped = NO;
    }
    else if (orientation == UIDeviceOrientationLandscapeLeft || orientation == UIDeviceOrientationLandscapeRight)
    {
        isLandScaped = YES;
        if (orientation == UIDeviceOrientationLandscapeLeft)
        {
            angle = M_PI_2;
        }
        else
        {
            angle = M_PI_2 * 3;
        }
    }
    else
    {
        if (self.isLandScaped != nil)
        {
            isLandScaped = [self.isLandScaped boolValue];
        }
        else
        {
            isLandScaped = YES;
        }
    }
    self.isFirstLaunch = NO;
    if (self.isLandScaped != nil && [self.isLandScaped boolValue] == isLandScaped)
    {
        return;
    }
    if (self.backGroundLayer != nil)
    {
        [self.backGroundLayer removeFromSuperlayer];
        self.backGroundLayer = nil;
    }
    self.isLandScaped = @(isLandScaped);
    self.borderRectInImage = CGRectZero;
    //根据像素大小和预览区域大小计算视频取景框内比例，用于识别框内信息
    CGFloat scale = self.imageFrameWidth / self.previewLayer.frame.size.height;
    if (isLandScaped)
    {
        if (screenWidth / screenHeight < GoldenSectionRation)
        {
            self.borderWidth = (int)(screenWidth * 7 / 8);
            self.borderHeight = self.borderWidth / GoldenSectionRation;
        }
        else
        {
            self.borderHeight = (int)(screenHeight * 7 / 8);
            self.borderWidth = self.borderHeight * GoldenSectionRation;
        }
    }
    else
    {
        if (screenHeight / screenWidth < GoldenSectionRation)
        {
            self.borderHeight = (int)(screenHeight * 7 / 8);
            self.borderWidth = self.borderHeight / GoldenSectionRation;
        }
        else
        {
            self.borderWidth = (int)(screenWidth * 7 / 8);
            self.borderHeight = self.borderWidth * GoldenSectionRation;
        }
    }
    
    //在视图(1920*1080)上取景区域x、y坐标轴，
    CGFloat originX = (self.imageFrameWidth - self.borderHeight * scale) / 2;
    CGFloat originY = (self.imageFrameHeight - self.borderWidth * scale) / 2;
    
    //在图片像素中需要识别的区域坐标信息
    self.borderRectInImage = CGRectMake(originX, originY, self.borderHeight * scale, self.borderWidth * scale);
    
    //在视图中显示在屏幕取景框尺寸坐标信息
    CGRect borderRect = CGRectMake((screenSize.width - self.borderWidth)/2, (screenSize.height - self.borderHeight)/2, self.borderWidth, self.borderHeight);
    
    
    //绘制背景图层
    UIBezierPath *superPath = [UIBezierPath bezierPathWithRoundedRect:self.view.bounds cornerRadius:0];
    [superPath setUsesEvenOddFillRule:YES];
    UIBezierPath *borderPath = [UIBezierPath bezierPathWithRoundedRect:borderRect cornerRadius:10];
    [borderPath setUsesEvenOddFillRule:YES];
    
    [superPath appendPath:borderPath];
    
    CAShapeLayer *fillLayer = [CAShapeLayer layer];
    fillLayer.path = superPath.CGPath;
    fillLayer.fillRule = kCAFillRuleEvenOdd;
    fillLayer.fillColor = [UIColor colorWithRed:0 green:0 blue:0 alpha:1].CGColor;
    fillLayer.opacity = 0.3;
    [self.view.layer addSublayer:fillLayer];
    self.backGroundLayer = fillLayer;
    
    //添加取景框内默认图片显示
    if (self.imageView == nil)
    {
        UIImageView *imageView = [[UIImageView alloc] init];
        NSString *imageName = @"card";
        if ([[NSLocale preferredLanguages] count] > 0)
        {
            NSString *currentLanguage = [[NSLocale preferredLanguages] objectAtIndex:0];
            if ([currentLanguage hasPrefix:@"zh-Hans"])
            {
                imageName = @"card_cn";
            }
            else if ([currentLanguage hasPrefix:@"zh-Hant"] || [currentLanguage hasPrefix:@"zh-TW"] || [currentLanguage hasPrefix:@"zh-HK"])
            {
                imageName = @"card_tw";
            }
        }
        [self.view addSubview:imageView];
        self.imageView = imageView;
    }
    
    
    //根据Home方位进行取景框旋转
    if (isLandScaped)
    {
        self.imageView.frame = CGRectMake(0, 0, borderRect.size.height, borderRect.size.width);
        self.imageView.center = CGPointMake(screenWidth / 2, screenHeight / 2);
        self.imageView.transform = CGAffineTransformMakeRotation(angle);
    }
    else
    {
        self.imageView.transform = CGAffineTransformMakeRotation(angle);
        self.imageView.frame = borderRect;
    }
}

- (void)setupOtherThings
{
    UITapGestureRecognizer *tapGesture = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(didTap:)];
    [self.view addGestureRecognizer:tapGesture];
}

- (void) didTap:(UITapGestureRecognizer *) tapGesture
{
    if (tapGesture.state == UIGestureRecognizerStateBegan)
    {
        [self focus];
    }
}

- (void)captureOutput:(AVCaptureOutput *)captureOutput didOutputSampleBuffer:(CMSampleBufferRef)sampleBuffer fromConnection:(AVCaptureConnection *)connection
{
    CFRetain(sampleBuffer);
    CGFloat scale = self.imageFrameWidth / self.previewLayer.frame.size.height;
    
//    CGRect rect = CGRectMake(self.borderView.frame.origin.y * scale, self.borderView.frame.origin.x * scale, self.borderView.frame.size.height * scale, self.borderView.frame.size.width * scale);
    
    [[ISIDCardReaderController sharedISOpenSDKController] detectCardWithOutputSampleBuffer:sampleBuffer cardRect:self.borderRectInImage detectCardFinishHandler:^(int result, NSArray *borderPointsArray) {
        if (result > 0)
        {
            dispatch_async(dispatch_get_main_queue(), ^{
                [self.cardBorderLayer removeFromSuperlayer];
                self.cardBorderLayer = nil;
                self.borderView.layer.borderWidth = 10.0f;
                [self drawBorderRectWithBorderPoints:borderPointsArray onCardBorderLayer:YES];
            });
        }
        else
        {
            dispatch_async(dispatch_get_main_queue(), ^
                           {
                               self.borderView.layer.borderWidth = 1.0f;
                               [self.cardBorderLayer removeFromSuperlayer];
                               self.cardBorderLayer = nil;
                           });
        }
    } recognizeCardFinishHandler:^(NSDictionary *cardInfo) {
        NSLog(@"%lu", (unsigned long)cardInfo.count);
        [self.captureSession stopRunning];
        dispatch_async(dispatch_get_main_queue(), ^{
            ISIDReaderResultViewController *vc = [[ISIDReaderResultViewController alloc] initWithDictionary:cardInfo];
            UIImage *originImage = [cardInfo objectForKey:kOpenSDKCardResultTypeOriginImage];
            UIImage *croppedImage = [cardInfo objectForKey:kOpenSDKCardResultTypeImage];
            NSLog(@"Result origin image %@, cropped image %@", originImage, croppedImage);
            [self presentViewController:vc animated:YES completion:^{
                
            }];
        });
    }];
    CFRelease(sampleBuffer);
}

- (void) alertView:(UIAlertView *)alertView didDismissWithButtonIndex:(NSInteger)buttonIndex
{
    [self.captureSession startRunning];
    [self.cardNumberLayer removeFromSuperlayer];
    self.cardNumberLayer = nil;
}

- (void)drawBorderRectWithBorderPoints:(NSArray *)borderPoints onCardBorderLayer:(BOOL)flag
{
    if ([borderPoints count] >= 4)
    {
        CGPoint point1 = [self convertBorderPointsInImageToPreviewLayer:[[borderPoints objectAtIndex:0] CGPointValue]];
        CGPoint point2 = [self convertBorderPointsInImageToPreviewLayer:[[borderPoints objectAtIndex:1] CGPointValue]];
        CGPoint point3 = [self convertBorderPointsInImageToPreviewLayer:[[borderPoints objectAtIndex:2] CGPointValue]];
        CGPoint point4 = [self convertBorderPointsInImageToPreviewLayer:[[borderPoints objectAtIndex:3] CGPointValue]];
        
        CAShapeLayer *line = [CAShapeLayer layer];
        UIBezierPath *linePath = [UIBezierPath bezierPath];
        [linePath moveToPoint:point1];
        [linePath addLineToPoint:point2];
        [linePath addLineToPoint:point3];
        [linePath addLineToPoint:point4];
        [linePath addLineToPoint:point1];
        line.path = linePath.CGPath;
        line.fillColor = nil;
        line.opacity = 1.0;
        line.strokeColor = [UIColor blueColor].CGColor;
        if (flag)
        {
            self.cardBorderLayer = line;
        }
        else
        {
            self.cardNumberLayer = line;
        }
        [self.previewLayer addSublayer:line];
    }
}

- (CGPoint)convertBorderPointsInImageToPreviewLayer:(CGPoint )borderPoint
{
    //video orientation is landscape
    CGFloat y = borderPoint.x * self.previewLayer.bounds.size.height / self.imageFrameWidth;
    CGFloat x = self.previewLayer.bounds.size.width - borderPoint.y * self.previewLayer.bounds.size.width / self.imageFrameHeight;
    return CGPointMake(x, y);
}

#define clamp(a) (a>255?255:(a<0?0:a));
//将视频流转换为图片
- (UIImage *) imageRefrenceFromSampleBuffer:(CMSampleBufferRef) sampleBuffer // Create a CGImageRef from sample buffer data
{
    CVImageBufferRef imageBuffer = CMSampleBufferGetImageBuffer(sampleBuffer);
    CVPixelBufferLockBaseAddress(imageBuffer,0);
    
    size_t width = CVPixelBufferGetWidth(imageBuffer);
    size_t height = CVPixelBufferGetHeight(imageBuffer);
    uint8_t *yBuffer = (uint8_t *)CVPixelBufferGetBaseAddressOfPlane(imageBuffer, 0);
    size_t yPitch = CVPixelBufferGetBytesPerRowOfPlane(imageBuffer, 0);
    uint8_t *cbCrBuffer = (uint8_t *)CVPixelBufferGetBaseAddressOfPlane(imageBuffer, 1);
    size_t cbCrPitch = CVPixelBufferGetBytesPerRowOfPlane(imageBuffer, 1);
    
    int bytesPerPixel = 4;
    uint8_t *rgbBuffer = (uint8_t *)malloc(width * height * bytesPerPixel);
    
    for(int y = 0; y < height; y++)
    {
        uint8_t *rgbBufferLine = &rgbBuffer[y * width * bytesPerPixel];
        uint8_t *yBufferLine = &yBuffer[y * yPitch];
        uint8_t *cbCrBufferLine = &cbCrBuffer[(y >> 1) * cbCrPitch];
        
        for(int x = 0; x < width; x++)
        {
            int16_t y = yBufferLine[x];
            int16_t cb = cbCrBufferLine[x & ~1] - 128;
            int16_t cr = cbCrBufferLine[x | 1] - 128;
            
            uint8_t *rgbOutput = &rgbBufferLine[x*bytesPerPixel];
            
            int16_t r = (int16_t)roundf( y + cr *  1.4 );
            int16_t g = (int16_t)roundf( y + cb * -0.343 + cr * -0.711 );
            int16_t b = (int16_t)roundf( y + cb *  1.765);
            
            rgbOutput[0] = 0xff;
            rgbOutput[1] = clamp(b);
            rgbOutput[2] = clamp(g);
            rgbOutput[3] = clamp(r);
        }
    }
    
    CGColorSpaceRef colorSpace = CGColorSpaceCreateDeviceRGB();
    CGContextRef context = CGBitmapContextCreate(rgbBuffer, width, height, 8, width * bytesPerPixel, colorSpace, kCGBitmapByteOrder32Little | kCGImageAlphaNoneSkipLast);
    CGImageRef quartzImage = CGBitmapContextCreateImage(context);
    UIImage *image = [UIImage imageWithCGImage:quartzImage];
    
    CGContextRelease(context);
    CGColorSpaceRelease(colorSpace);
    CGImageRelease(quartzImage);
    free(rgbBuffer);
    
    CVPixelBufferUnlockBaseAddress(imageBuffer, 0);
    
    return image;
}
@end
